/**
 *
 */
package jp.agentec.sinaburocast.action.admin.member;

import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.IOUtils;
import org.apache.log4j.Logger;
import org.apache.struts.util.TokenProcessor;
import org.seasar.framework.util.FileUtil;
import org.seasar.framework.util.StringUtil;
import org.seasar.struts.annotation.ActionForm;
import org.seasar.struts.annotation.Execute;
import org.seasar.struts.util.ResponseUtil;

import jp.agentec.sinaburocast.action.AbstractAction;
import jp.agentec.sinaburocast.common.SinaburoConstant;
import jp.agentec.sinaburocast.common.annotation.OperationHistoryAno;
import jp.agentec.sinaburocast.common.util.PropertyUtil;
import jp.agentec.sinaburocast.common.util.SinaburoUtil;
import jp.agentec.sinaburocast.common.util.SinaburoViewUtil;
import jp.agentec.sinaburocast.csv.MemberInfoCsv;
import jp.agentec.sinaburocast.entity.UploadError;
import jp.agentec.sinaburocast.form.admin.member.MemberRegistAllForm;
import jp.agentec.sinaburocast.service.MemberService;
import jp.agentec.sinaburocast.service.UploadErrorService;
import jp.agentec.sinaburocast.service.csv.CsvService;

/**
 * @author kim-hs
 *
 * 会員情報一括登録
 *
 */
public class MemberRegistAllAction extends AbstractAction {

	private final Logger logger = Logger.getLogger(getClass());

	private final String header = "\"会員ID\"\t\"ログインID\"\t\"パスワード\"\t\"氏\"\t\"名\"\t\"氏(カナ)\"\t\"名(カナ)\"\t\"E-mail(メイン)\"\t\"E-mail(サブ)\"\t\"電話\"\t\"性別\"\t\""+
			"生年月日\"\t\"年代\"\t\"郵便番号\"\t\"都道府県\"\t\"市区町村\"\t\"町名・番地\"\t\"建物名\"\t\"ポイント\"\t\"メモ\"\t\"削除フラグ\"\t\"退会有無\""+"\r\n";

	@ActionForm
	public MemberRegistAllForm memberRegistAllForm;

	public UploadErrorService uploadErrorService;

	public MemberService memberService;

	//ファイルタイプID
	private final String type="1";
	/**
	 * ジェネリクスを変えることでCSV関連Entity対応出来る。
	 */
	public CsvService<MemberInfoCsv> csvService;

	/**
	 * 初期表示
	 */
	@Execute(validator = false)
	public String index() {
		//エラーファイルダウンロード
		memberRegistAllForm.uploadError = uploadErrorService.findByType(type);
    	// トークン発行
		TokenProcessor.getInstance().saveToken(request);

		return "/admin/member/memberInputAll.html";
	}

	/**
	 * 会員一括登録完了
	 */
	@Execute(validator = false)
	public String memberRegistAll() {
		//エラーファイルダウンロード
		memberRegistAllForm.uploadError = uploadErrorService.findByType(type);
		return "/admin/member/memberRegistAll.html";
	}

	/**
	 * 会員一括登録サンプルCSVダウンロード
	 */
	@Execute(validator=false)
	@OperationHistoryAno(operationTypeId=SinaburoConstant.OperationType.DOWNLOAD,target="会員一括登録用",detail="ダウンロード",screenId=SinaburoConstant.screenId.CMM0006)
	public String download() throws UnsupportedEncodingException, Exception {

		StringBuffer str = new StringBuffer();
		str.append(header);

		//20170508 SJIS文字化け対応
		String outStr = str.toString();
		outStr = SinaburoUtil.toSJIS(outStr);
		ResponseUtil.download(URLEncoder.encode("member_upload.tsv", "UTF-8"), outStr.getBytes("Windows-31J"));
		//ResponseUtil.download(URLEncoder.encode("member_upload.tsv", "UTF-8"), str.toString().getBytes("Shift_JIS"));
		//response.setContentType("Content-Type content=\"text/html; charset=\"UTF-8\"");
		return null;

	}

	/**
	 * 会員一括登録
	 */
	@Execute(validator = true, input="/admin/member/memberRegistAll/")
	@OperationHistoryAno(operationTypeId=SinaburoConstant.OperationType.UPLOAD,target="会員一括登録",detail="アップロード",screenId=SinaburoConstant.screenId.CMM0006)
	public String memberUpload() throws Exception {

    	// トークン確認
   		if (!TokenProcessor.getInstance().isTokenValid(request, true)) {

   			return "/error/doubleAction.html";
    	}

		int result = 0;
		String csvData = null;
		InputStream in = null;

		if(!memberRegistAllForm.file.getFileName().toUpperCase().endsWith(".TSV")){
			addError("errors.E040");
			return "/admin/member/memberRegistAll/";
		}

		try {
			in = memberRegistAllForm.file.getInputStream();
			//バイナリ形式チェック
            byte[] b = new byte[1];
            while (in.read(b, 0, 1) > 0) {
                if (b[0] == 0) {
        			addError("errors.E042");
        			return "/admin/member/memberRegistAll/";
                }
            }

		} finally {
			IOUtils.closeQuietly(in);
		}

		try {
			in = memberRegistAllForm.file.getInputStream();
			//20170508 SJIS文字化け対応
			csvData = IOUtils.toString(in, "Windows-31J");
			//csvData = IOUtils.toString(in, "Shift_JIS");
		} finally {
			IOUtils.closeQuietly(in);
		}

		StringBuffer isOk   = new StringBuffer("OK");
		//List<MemberInfoCsv> memberCsvList = csvService.parseAll(csvData, MemberInfoCsv.class,isOk);
		List<MemberInfoCsv> memberCsvList = null;

		try {
			memberCsvList = csvService.parseAll(csvData, MemberInfoCsv.class,isOk);
		} catch (org.seasar.s2csv.csv.exception.CSVFormatException e) {
			addError("errors.E039");
			return "/admin/member/memberRegistAll/";
		}catch(NumberFormatException e){
			addError("errors.E039");
			return "/admin/member/memberRegistAll/";
		}

		if (CollectionUtils.isEmpty(memberCsvList)) {
			addError(SinaburoViewUtil.getMessage("errors.E038"));
			return "/admin/member/memberRegistAll/";
		}

		/**結果登録*/
		UploadError uploadError = new UploadError();
		uploadError.filePath = PropertyUtil.getProperty(SinaburoConstant.filePath.UPLOAD_ERROR_FILE_PATH);
		uploadError.filePath+="member_upload_error.tsv";
		uploadError.type=type;//市民団体一括アップロード
		uploadError.status=  StringUtil.equals(isOk.toString(), "NG") ? "NG" : "OK";
		uploadErrorService.deleteInsertByType(uploadError,getAdmin());
		memberRegistAllForm.uploadError = uploadErrorService.findByType(type);
		/**結果ファイル作成*/

		//20170508 SJIS文字化け対応
		String outStr = SinaburoUtil.addDoubleStr(csvService.getContents(memberCsvList, MemberInfoCsv.class).replace("\"", ""));
		outStr = SinaburoUtil.toSJIS(outStr);
		FileUtil.write(uploadError.filePath, outStr.getBytes("Windows-31J"));
		//FileUtil.write(uploadError.filePath, SinaburoUtil.addDoubleStr(csvService.getContents(memberCsvList, MemberInfoCsv.class).replace("\"", "")).getBytes("Shift_JIS"));

		/**データチェックエラー(1件でも)時には更新を行わない*/
		if(StringUtil.equals(isOk.toString(), "NG")){
	    	// トークン発行
			TokenProcessor.getInstance().saveToken(request);
			addError(SinaburoViewUtil.getMessage("errors.E036"));
			return "/admin/member/memberInputAll.html";
		}

		/**一括登録*/
		result = memberService.memberAllRegist(memberCsvList, getAdmin());
		logger.info("一括更新件数："+result);

		return "/admin/member/memberRegistAll.html";

	}

	/**
	 * 最後に登録したファイルの結果ファイルをダウンロード
	 * @throws Exception
	 */
	@Execute(validator = false)
	@OperationHistoryAno(operationTypeId=SinaburoConstant.OperationType.DOWNLOAD,target="会員一括登録エラー",detail="エラーダウンロード",screenId=SinaburoConstant.screenId.CMM0006)
	public String resultFileDownLoad()throws Exception {

		 UploadError uploadError = uploadErrorService.findByType(type);
		 jp.agentec.sinaburocast.common.io.FileUtil.fileDownload(uploadError.filePath, "Shift_JIS");

		 memberRegistAllForm.uploadError = uploadErrorService.findByType(type);

		return null;
	}

}
