package jp.agentec.sinaburocast.service.csv;

import java.io.StringReader;
import java.io.StringWriter;
import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.List;

import javax.annotation.Resource;

import jp.agentec.sinaburocast.common.util.SinaburoUtil;
import jp.agentec.sinaburocast.csv.AbstractCsv;

import org.seasar.framework.util.ClassUtil;
import org.seasar.framework.util.FieldUtil;
import org.seasar.s2csv.csv.S2CSVParseCtrl;
import org.seasar.s2csv.csv.S2CSVWriteCtrl;
import org.seasar.s2csv.csv.annotation.column.CSVColumn;
import org.seasar.s2csv.csv.annotation.entity.CSVEntity;
import org.seasar.s2csv.csv.exception.runtime.CSVValidationResultException;
import org.seasar.s2csv.csv.factory.S2CSVCtrlFactory;
import org.seasar.s2csv.csv.message.CSVMsg;
import org.seasar.s2csv.csv.validator.CSVValidateResult;

/**
 * CSVをパースして{@link DeptCsv}のリストを返すサービスです。
 *
 * @author choi-c
 * ジェネリクスを変えることでCSV関連Entity対応出来る。
 */
public class CsvService<T extends AbstractCsv> {

	@Resource
	public S2CSVCtrlFactory s2csvCtrlFactory;


	/**
	 * CSVを一括パースします。
	 *
	 * @param csvData CSVデータ
	 * @return CSVをパースして作成されたclazzのクラスのリスト
	 * @throws IllegalAccessException
	 * @throws InstantiationException
	 */
	public List<T> parseAll(String csvData, Class<T> clazz,StringBuffer isOk) throws InstantiationException, IllegalAccessException{
		try {

			S2CSVParseCtrl<T> controller = (S2CSVParseCtrl<T>) s2csvCtrlFactory.getParseController(
				clazz, new StringReader(csvData));

			return controller.parseAll();

		} catch(CSVValidationResultException ex){
			List<CSVValidateResult> resultList = ex.getValidateResultList();
			List<T> dataList = ex.getDataList();

			Field [] fields = clazz.getFields();
			HashMap<Integer, String> fieldMap = new HashMap<Integer, String>();
			for(int i=0;i<fields.length;i++){
				//fields[i].getName();
				CSVColumn csvColumn = fields[i].getAnnotation(CSVColumn.class);
				if(csvColumn == null){
					continue;
				}
				fieldMap.put(csvColumn.columnIndex(),fields[i].getName());
			}



			CSVEntity anotation = clazz.getAnnotation(CSVEntity.class);
			int headerindex = anotation.header() ? -1 : 0 ;


			for(CSVValidateResult result: resultList){
				//NGになったCSVデータを復元する。
				T t = clazz.newInstance();
				String [] data = result.getData();
				Field field = null;
				for(int j=0 ;j < data.length ;j++){
					field = ClassUtil.getField(clazz, fieldMap.get(j));
					/*○○CsvエンティティにはStringとIntegerだけが宣言できる*/
					if(field.getGenericType() == Integer.class){
						FieldUtil.set(field,t,Integer.parseInt(data[j]));
					}else{
						FieldUtil.set(field,t,data[j]);
					}
				}

				//結果メッセージ設定。
				field = ClassUtil.getField(clazz, "okNg");
				FieldUtil.set(field,t,"NG");

				int ctn = 0;
				//理由は複数あった場合その数分追加する。
				for(CSVMsg msg: result.getMsgs()){

					field = ClassUtil.getField(clazz, "reason");
					Object o =FieldUtil.get(field, t);
					if (ctn == 0) {
						o = null;
					}
					ctn++;
					//既にメッセージがあった場合は追加する。
					if(o == null){
						FieldUtil.set(field,t,msg.toString());
					}else{
						FieldUtil.set(field,t,o+msg.toString());
					}
				}

				dataList.add((int)result.getLineNo() +headerindex -1, t);
			}
			isOk.delete(0, isOk.length());
			isOk.append("NG");
			return dataList;
		}
	}


	public void csvOutputAll(List<T> csvDataList,Class<T> clazz) throws Exception{
		StringWriter writer = new StringWriter();
		S2CSVWriteCtrl<T> controller
			= s2csvCtrlFactory.getWriteController(clazz, writer);

		controller.writeAll(csvDataList);
		String a = writer.toString();
		SinaburoUtil.writeFile("c:/testCsv2.txt", a, "Shift_JIS");
	}

	public byte[] getByte(List<T> csvDataList,Class<T> clazz) throws Exception{
		StringWriter writer = new StringWriter();
		S2CSVWriteCtrl<T> controller
			= s2csvCtrlFactory.getWriteController(clazz, writer);
		controller.writeAll(csvDataList);

		return writer.toString().getBytes("Shift_JIS");

		//SinaburoUtil.writeFile("c:/testCsv2.txt", a, "Shift_JIS");
	}

	public String getContents(List<T> csvDataList,Class<T> clazz) throws Exception{
		StringWriter writer = new StringWriter();
		S2CSVWriteCtrl<T> controller
			= s2csvCtrlFactory.getWriteController(clazz, writer);

		controller.writeAll(csvDataList);
		return writer.toString();

	}


//	/**
//	 * CSVを1件ずつパースします。
//	 *
//	 * @param csvData CSVデータ
//	 * @param errors エラーがあった場合、このActionErrorsオブジェクトにActionMessageが追加されます。
//	 * @return CSVをパースして作成されたDeptCsvのリスト
//	 */
//	public List<T> parse(String csvData, ActionErrors errors,Class<T> clazz){
//		List<T> deptList = new ArrayList<T>();
//
//		S2CSVParseCtrl<T> controller = s2csvCtrlFactory.getParseController(
//			clazz, new StringReader(csvData));
//
//		while(controller.readNext()){
//			try {
//				T clzz = controller.parse();
//				deptList.add(clzz);
//			} catch(CSVValidationResultRuntimeException ex){
//				CSVValidateResult result = ex.getValidateResult();
//				for(CSVMsg msg: result.getMsgs()){
//					errors.add("csvData", new ActionMessage(
//							result.getLineNo() + "行目：" + msg.toString(), false));
//				}
//			}
//		}
//		return deptList;
//	}



}